@@ -1,9 +1,9 @@
-/**************************************************************
-                  June 21, 2002  Shin Satoh (satoh@crl.go.jp)
+/**********************************************************
+                  Jan. 21, 2002  Shin Satoh (satoh@crl.go.jp)
 
 Main routine of the TRMM PR Latent Heating Algorithm
 
-SOURCE FILE: glhret_v3.3.c  <<< version 3.3 >>>
+SOURCE FILE: glhret_v3.1.c  <<< version 3.1 >>>
 PURPOSE:     Retreive the latent heating profiles using 
              a TRMM 2A25 product
 INPUT DATA:  2A25 version 5 swath data
@@ -13,10 +13,10 @@ COMPILE:     make -f glhret.make
 USAGE:       glhret {Input file name} {GrADS Output file name}\ 
                      [start scan num] [end scan num]
              (it is possible to omit [] terms)
-NOTE:        You need "param_lhret.dat" and "trop_ht.dat"
-             files on your executing directory
+NOTE:        You need "param_lhret.dat" file on your 
+             executing directory
 
-****************************************************************/
+***************************************************************/
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -24,7 +24,7 @@ NOTE:        You need "param_lhret.dat" and "trop_ht.dat"
 #include "IO.h"             /* TSDIS Tool Kit */
 #include "IO_PR.h"          /* TSDIS Tool Kit */
 #include "IO_INTR_PR.h"     /* TSDIS Tool Kit */
-#include "glhret_v3.3.h"    /* defines function prototypes */
+#include "glhret_v3.0.h"    /* defines function prototypes */
 
 #define  YDIM      49      /* number of angle bins in output */
 #define  ZDIM      80      /* number of vertical bins in output */
@@ -59,7 +59,7 @@ int main(int argc, char *argv[])
 
 /* ------- Common variables ------- */
     int   sscan,escan;
-    int   i,ii,j,k,kk,l,lp1,irt;
+    int   i,ii,j,k,kk,k1,k2,l,lp1,irt;
     int   ihr,imin,isec;
     float stime,slat,slon;
     float first_scan_sec=0.0;
@@ -88,48 +88,40 @@ int main(int argc, char *argv[])
     int   raintype;                    /* rain type (0=STRAT, 1=CONV, 2=OTHERS) */ 
     int   flag_warm;                   /* warm rain flag */
 
-/* ------- 2A25 original/derived variable ------- */
+/* ------- 2A25 original/derived variable ------- */ 
     int   sfcBin, bbBin, cldtBin;
     int   rbinNum[N6];                 /* range bin numbers of top[0], bottom[1], 
                                           surface[2], BB[3], PIZ_threshold[4], ZeMax[5] */
-    int   rbupTop, rbupBottom, rbupSurf, rbupBB;
-    int   kcldt, kcldb, lowest_flag;
+    int   rbfsTop, rbfsBottom, rbfsSurf, rbfsBB;
     int   rbinNode[N5];                /* node range bin numbers of ZR coefficients */
     float epsilonF;                    /* attenuation correction factor */
     float weightF;                     /* weighting factor in the calc of epsilon */
     float nubfCF;                      /* non-uniform beam filling correction factor */
     double xze, xz, xrain, xwc, xqp, xvt, xvt0;
-    double rainsum, rainnum;
-    double qpsum, qpnum, qpdiff, vtsum, vtnum;
-    /* original 2-dims array is defined in the c-lang order, i.e rain[YDIM][ZDIM],
-       but this program uses the opposite arangement order for GrADS output */
     static float  cze[ZDIM][YDIM];
     static float  rain[ZDIM][YDIM];
     static float  wc[ZDIM][YDIM];
-    static float  qp[ZDIM][YDIM];
+    static float  qp[ZDIM][YDIM], xxqp[ZDIM][YDIM];
     static float  vt[ZDIM][YDIM];
     static float  vt0[ZDIM][YDIM];
     static float  zra[ZDIM][YDIM], zrb[ZDIM][YDIM];
     static float  zwa[ZDIM][YDIM], zwb[ZDIM][YDIM];
-    static float  xxrain[ZDIM], xxqp[ZDIM], xxvt[ZDIM];
     static float  flag_rain[YDIM];  
     static float  flag_PIA[YDIM];
     static float  flag_rtype[YDIM];  
     static float  flag_BB[YDIM];
     static float  flag_bottom[YDIM];
-    static float  flag_wtype[YDIM], xxflag[YDIM];
+    static float  flag_wtype[YDIM];
     static float  sfcr[YDIM];
     static float  sfcz[YDIM];
     static float  sfcht[YDIM];         /* bright band height [km] */
     static float  bbht[YDIM];          /* surface height [km] */
-    static int    rbinBottom[YDIM];    /* same as rbinNum[1]: bottom range bin */
 
 /* ------- latent heating output ------- */ 
     int   iteration;
     static float  trop[YDIM];          /* troppause height [km] */
     static float  cldt[YDIM];          /* cloud top height [km] */
     static float  cldb[YDIM];          /* cloud bottom height [km] */
-    static float  avgsfcr[YDIM];       /* averaged near surface rain [mm/hr] */ 
     static float  w[ZDIM][YDIM];       /* vertical wind profile [m/s] */
     static float  fqp[ZDIM][YDIM];     /* production rate of qp [kg/s] */
     static float  lh[ZDIM][YDIM];      /* latent heating profile [K/hr] */
@@ -262,7 +254,21 @@ int main(int argc, char *argv[])
                rbinNode[l] = (int)read_data.ZRParmNode[j][l];
             }
 
-            /*--- get rainfall flag in 2A25 ------------------------------
+            /*--- get surface & BB height [km] --------------------------
+               range bin numbers of top[0], bottom[1],surface[2], BB[3],
+                                    PIZ_threshold[4], ZeMax[5]
+	    -------------------------------------------------------------*/
+            for(l=0; l<N6; l++){
+               rbinNum[l] = (int)read_data.rangeBinNum[j][l];
+            }
+            /* cldt[j]=(79-rbinNum[0])*ZRESO;    /* 1 km above the rain-certain bin */
+            /* cldb[j]=(79-rbinNum[1])*ZRESO;    /* bottom range bin (+1) height */
+            /* ---> cldt & cldb are defined by original cld_ht routine */
+            sfcht[j] = (79-rbinNum[2])*ZRESO;    /* actual surface height */
+            bbht[j]  = (79-rbinNum[3])*ZRESO;    /* bright band height */
+
+	   
+            /*--- rainfall flag in 2A25 ------------------------------
 	          0: (bit  1) no rain
 	   	 +1: (bit  1) rain possible
 	   	 +2: (bit  2) rain certain
@@ -297,54 +303,33 @@ int main(int argc, char *argv[])
             if(((read_data.rainFlag[j]&512)>>9)==1)    flag_bottom[j]=2.0;
             if(((read_data.rainFlag[j]&16384)>>14)==1) flag_bottom[j]=3.0;
 
-            /*--- get surface & BB height [km] --------------------------
-               range bin numbers of top[0], bottom[1],surface[2], BB[3],
-                                    PIZ_threshold[4], ZeMax[5]
-	    -------------------------------------------------------------*/
-            for(l=0; l<N6; l++){
-               rbinNum[l] = (int)read_data.rangeBinNum[j][l];
-            }
-            rbupTop    = 79-rbinNum[0];     /* Top range bin num upward */
-            rbupBottom = 79-(rbinNum[1]-1); /* Bottom range bin num upward */
-            rbupSurf   = 79-rbinNum[2];     /* Surf range bin num upward */
-            rbupBB     = 79-rbinNum[3];     /* BB range bin num upward */
-            /* Memo: - rbinNum[0] is defined as the 1 km above the 
-                       rain-certain bin (in ver. 5.53), but sometimes 
-                       it may be the highest obs range bin even if it 
-                       is no-rain.
-	             - rbinNum[1] is the top of the possible surface
-                       cluttered range. Hence, "rbinNum[1]-1" is 
-                       the actual valid rain bottom data. The near-
-                       surface-rain must be the same as rain[rbinNum[1]-1]
-            */
-            cldt[j] = rbupTop*ZRESO;      /* temporal(initial) cloud top height */
-            cldb[j] = rbupBottom*ZRESO;   /* temporal(initial) cloud bottom height */
-            sfcht[j] = rbupSurf*ZRESO;    /* actual surface height */
-            bbht[j]  = rbupBB*ZRESO;      /* bright band height */
-	    rbinBottom[j] = rbinNum[1];   /* store to get a new surface rain later */
-
             /*--- new flag (flag_wtype) to choice the w-profile shape ----
                 flag_wtype 
                 0.=NO_RAIN, 1.=CONV, 2.=STRAT, 3.=ANVIL, 4.=SHALLOW
  	    -------------------------------------------------------------*/
-            /* if(flag_rain[j] > 0.9){                      /* rain possible */
+            rbfsTop    = 79-rbinNum[0];   /* Top range bin from surface */
+            rbfsBottom = 79-rbinNum[1];   /* Bottom range bin from surface */
+            rbfsSurf   = 79-rbinNum[2];   /* Surf range bin from surface */
+            rbfsBB     = 79-rbinNum[3];   /* BB range bin from surface */
+            /* if(flag_rain[j] > 0.9){                         /* rain possible */
             if(flag_rain[j] > 1.9){                         /* rain certain */
                if(flag_rtype[j]<0.1){                       /* original strat */
-                  if(rbupTop>=rbupBB) flag_wtype[j]=2.0;  
-                  if(rbupTop<rbupBB)  flag_wtype[j]=4.0;
+                  if(rbfsTop>=rbfsBB) flag_wtype[j]=2.0;  
+                  if(rbfsTop<rbfsBB)  flag_wtype[j]=4.0;
                }
                if(0.9<flag_rtype[j] && flag_rtype[j]<1.1){  /* original conv */
 		  flag_wtype[j]=1.0;
                }
                if(1.9<flag_rtype[j] && flag_rtype[j]<2.1){  /* original others */
                   flag_wtype[j]=3.0;
-                  if(rbupTop<rbupBB)  flag_wtype[j]=4.0;
-                  /* if(rbupBottom>rbupSurf) flag_wtype[j]=3.0; --- typical, but no need */
+                  if(rbfsTop<rbfsBB)  flag_wtype[j]=4.0;
+                  /* if(rbfsBottom>rbfsSurf) flag_wtype[j]=3.0; --- typical, but no need */
                }
             }else{
 	       flag_wtype[j]=0.0;  /* no_rain */
             }
 
+
             /*--- raintype (-1=NO_RAIN, 0=STRAT, 1=CONV, 2=OTHERS) ------
                                                       for Z-WC relation 
                 in this routine, rain possible & others => STRAT (anvil)
@@ -398,7 +383,7 @@ int main(int argc, char *argv[])
 	    /*                             clutter value (-88.9)              */
             /*----------------------------------------------------------------*/
             for(k=0; k<NRANGE; k++){
-               kk=NRANGE-1-k;                          /* the same as 79-k */
+               kk=NRANGE-1-k;                  /* the same as 79-k */
                xrain = (float)read_data.rain[j][k];             /* [mm/hr] */
                if(xrain < 0.0) xrain = 0.0;
                xze  = (float)read_data.correctZFactor[j][k];    /* [dBZ] */
@@ -430,189 +415,51 @@ int main(int argc, char *argv[])
 	    }
 
 
-            if(first_scan_sec==0.0) first_scan_sec = stime;
-            if(j==24 && i==sscan){       /* on nadir */
-               ihr =(int)(stime/3600.);
-               imin=(int)((stime-ihr*3600.)/60.);
-               isec=(int)(stime-ihr*3600.-imin*60.);
-               printf("First scan num: %4d (t=%4d)  %2d:%2d:%2d  %7.3f,%8.3f (at nadir)\n",
-                       i,i-sscan,ihr,imin,isec,slat,slon);
-            }
-            if(j==24 && i==escan){       /* on nadir */
-               ihr =(int)(stime/3600.);
-               imin=(int)((stime-ihr*3600.)/60.);
-               isec=(int)(stime-ihr*3600.-imin*60.);
-               printf("Last scan num: %4d (t=%4d)  %2d:%2d:%2d  %7.3f,%8.3f (at nadir)\n",
-                       i,i-sscan,ihr,imin,isec,slat,slon);
-            }
+           if(first_scan_sec==0.0) first_scan_sec = stime;
+           /* if(j==24 && (i==sscan || i==escan)){       /* on nadir */
+           if(j==24){       /* on nadir */
+             ihr =(int)(stime/3600.);
+             imin=(int)((stime-ihr*3600.)/60.);
+             isec=(int)(stime-ihr*3600.-imin*60.);
+             printf("Scan:%4d  %2d:%2d:%2d  %7.3f,%8.3f \n",
+                     i,ihr,imin,isec,slat,slon);
+           }
 
          }  /* end of angle loop */
          }  /* if-else of readScan error */
 
 /* ===========================================================================
-      GET CLOUD TOP and BOTTOM HEIGHTS
+      ESTIMATE w-, Fqp-, LH-profile
    =========================================================================== */
          /*--- get cloud top and bottom height ---*/
-         status = cld_ht(cze, flag_wtype, cldt, cldb, trop);
-         for(j=0; j<YDIM; j++){
-            if(sfcht[j]>8.848){     /* abnormal surface height [km] */ 
-               sfcht[j]=0.0;
-               bbht[j]=0.0;
-               cldb[j]=0.0;
-               flag_wtype[j]=0.0;
-            }
-            if(cldt[j]>19.75) cldt[j]=19.75;
-            if(cldt[j]<0.0)   cldt[j]=0.0;
-            if(cldb[j]>19.75) cldb[j]=19.75;
-            if(cldb[j]<0.0)   cldt[j]=0.0;
+         status = cld_ht(cze, flag_rain, flag_rtype, rbinNum,
+                         cldt, cldb, trop);
 
-            if(cldt[j]<=sfcht[j]) flag_wtype[j]=0.0;
-            if(cldt[j]<=bbht[j])  cldt[j]=bbht[j]+ZRESO;
-            if(trop[j]<=cldt[j])  trop[j]=cldt[j]+ZRESO;
-            if(cldb[j]<=sfcht[j]) cldb[j]=sfcht[j]+ZRESO;
-         }
-
-         /*--- change wtype from stratiform into convective 
-	       in an isolated stratiform with heavy rainfall
-               0.=NO_RAIN, 1.=CONV, 2.=STRAT, 3.=ANVIL, 4.=SHALLOW
- 	 -------------------------------------------------------------
+         /*--- smoothing ---*/
          for(j=0; j<YDIM; j++){
-           xxflag[j]=flag_wtype[j];
-         }
-         for(j=1; j<YDIM-1; j++){
-           if(flag_wtype[j]==2.0 &&
-              flag_wtype[j-1]==1.0 && flag_wtype[j+1]==1.0){
-              if(sfcr[j] > 1.0){
-                xxflag[j]=1.0;
-              }
-              printf("## j=%2d  flag_wtype=%f  new_falg=%f  surfR=%f\n",
-			j, flag_wtype[j], xxflag[j], sfcr[j]);
-#ifdef DEBUG3
-              printf("## j=%2d  flag_wtype=%f  new_falg=%f  surfR=%f\n",
-			j, flag_wtype[j], xxflag[j], sfcr[j]);
-#endif
-           }
-         }
-         for(j=0; j<YDIM; j++){
-           flag_wtype[j]=xxflag[j];
-         }
-         */
-
-/* ===========================================================================
-      ELIMINATE UNNECESSARY qp and AVERAGE qp and vt
-   =========================================================================== */
-         for(j=0; j<YDIM; j++){
-	   kcldt = cldt[j]/ZRESO;    /* 0 <= kcldt <= 79 */
-	   kcldb = cldb[j]/ZRESO;
-           /*--- eliminate qp above the cldt, under the cldb---*/
-           for(k=0; k<kcldb+1; k++){
-             qp[k][j]=UNDEF;
-           }
-           for(k=kcldt; k<ZDIM; k++){
-             qp[k][j]=UNDEF;
-           }
-
-           /*--- eliminate BB of qp using the 5 bins average of qp ---*/
            for(k=0; k<ZDIM; k++){
-             xxqp[k]=qp[k][j];
+             xxqp[k][j]=qp[k][j];
+             qp[k][j]=0.0;
            }
-           for(k=kcldb+2; k<kcldt-2; k++){
-             qpsum=0.0;  qpnum=0.0;
-             for(kk=k-2; kk<k+3; kk++){
-               if(xxqp[kk]>0.0){
-                 qpsum += xxqp[kk];
-                 qpnum += 1.0;
-               }
-             }
-             if(qpnum>1.0){
-               qpdiff = (xxqp[k]-(qpsum/qpnum))*1000.0;  /* [g/kg] */
-               if(qpdiff>0.2){
-                 qp[k][j]=qpsum/qpnum;
-#ifdef DEBUG3
-                 printf("## j=%2d  k=%2d  xxqp=%f  avg=%f  diff=%f\n",
-			j, k, xxqp[k], qpsum/qpnum, qpdiff);
-#endif
-               }
-             }
+           for(k=1; k<ZDIM-1; k++){
+             qp[k][j]=(xxqp[k-1][j]+xxqp[k][j]+xxqp[k+1][j])/3.0;
            }
-
-           /*--- 5 bins averaging of rain, qp and vt ---*/
+         }
+         for(j=0; j<YDIM; j++){
            for(k=0; k<ZDIM; k++){
-             xxrain[k]=rain[k][j];
-             xxqp[k]=qp[k][j];
-             xxvt[k]=vt[k][j];
+             xxqp[k][j]=qp[k][j];
+             qp[k][j]=0.0;
            }
-           for(k=kcldb+2; k<kcldt-2; k++){
-             rainsum=0.0;  rainnum=0.0;
-             qpsum=0.0;    qpnum=0.0;
-             vtsum=0.0;    vtnum=0.0;
-             for(kk=k-2; kk<k+3; kk++){
-               if(xxrain[kk]>0.0 && xxqp[kk]>0.0){
-                 rainsum += xxrain[kk];
-                 rainnum += 1.0;
-                 qpsum += xxqp[kk];
-                 qpnum += 1.0;
-               }
-               if(xxvt[kk]>UNDEF){
-                 vtsum += xxvt[kk];
-                 vtnum += 1.0;
-               }
-             }
-             if(qpnum>1.0) qp[k][j]=qpsum/qpnum;
-             if(rainnum>1.0) vt[k][j]=-1.0*(rainsum/rainnum)/(3600.0*qp[k][j]*rho[k]);
-             /* if(vtnum>1.0) vt[k][j]=vtsum/vtnum; */
-#ifdef DEBUG3
-             printf("## j=%2d  k=%2d  qp=%f %f  qpsum=%f %f  vt=%f %f  vtsum=%f %f\n",
-                     j, k, qp[kk][j], xxqp[kk], qpsum, qpnum, 
-		    vt[kk][j], xxvt[kk], vtsum, vtnum);
-#endif
+           for(k=1; k<ZDIM-1; k++){
+             qp[k][j]=(xxqp[k-1][j]+xxqp[k][j]+xxqp[k+1][j])/3.0;
            }
+         }
 
-           /*--- fill lowest/2nd-lowest qp and vt ---*/
-           lowest_flag=0;
-           for(k=0; k<ZDIM-3; k++){
-              if(lowest_flag==0 &&
-                 qp[k+2][j]>UNDEF && qp[k+3][j]>UNDEF){
-	            qp[k+1][j] = qp[k+3][j] + 2.0*(qp[k+2][j]-qp[k+3][j]);
-	            qp[k][j] =   qp[k+3][j] + 3.0*(qp[k+2][j]-qp[k+3][j]);
-	            vt[k+1][j] = vt[k+3][j] + 2.0*(vt[k+2][j]-vt[k+3][j]);
-	            vt[k][j] =   vt[k+3][j] + 3.0*(vt[k+2][j]-vt[k+3][j]);
-                    lowest_flag=1;
-              }
-           } 
-
-	   /*--- make a new sfcr using averaged qp and vt ---*/
-           kk = 79-(rbinBottom[j]-1);        /* kk is the upward range bin */
-           if(vt[kk][j]>UNDEF && qp[kk][j]>UNDEF){
-              avgsfcr[j] = 3600. * (-1.0) * vt[kk][j] * qp[kk][j] * rho[kk]; 
-           }else{
-	      avgsfcr[j] = 0.0;
-           }
-
-
-         }  /* end of j-loop (j<YDIM) */
-
-/* ===========================================================================
-      RETRIEVE w-, Fqp-, LH-profile
-   =========================================================================== */
          /*--- retrieve LH profile ---*/
          iteration = lh_prof(rho, qvs, flag_wtype, sfcr,    
 			     sfcht, bbht, cldt, cldb, trop, 
                              qp, vt, vt0, w, fqp, lh);
-         /* iteration = lh_prof(rho, qvs, flag_wtype, avgsfcr,    
-		  	        sfcht, bbht, cldt, cldb, trop, 
-                                qp, vt, vt0, w, fqp, lh);  --- use org sfcr */
 
-         if(iteration>28){ 
-           ihr =(int)(stime/3600.);
-           imin=(int)((stime-ihr*3600.)/60.);
-           isec=(int)(stime-ihr*3600.-imin*60.);
-           if(iteration==99) iteration=0;
-           printf("------ the above failure (max itar=%2d) occured in the scan: ",
-                   iteration);
-           printf("%4d (t=%4d)  %2d:%2d:%2d  lat=%7.3f lon=%8.3f ------\n",
-                   i,i-sscan,ihr,imin,isec,slat,slon);
-         }
 
 /* ===========================================================================
       OUTPUT DATA
@@ -631,7 +478,6 @@ int main(int argc, char *argv[])
           fwrite(lh,1,outsize2,fpg);
           fwrite(sfcz,1,outsize1,fpg);
           fwrite(sfcr,1,outsize1,fpg);
-          fwrite(avgsfcr,1,outsize1,fpg);
           fwrite(sfcht,1,outsize1,fpg);
           fwrite(bbht,1,outsize1,fpg);
           fwrite(cldt,1,outsize1,fpg);
@@ -654,11 +500,11 @@ int main(int argc, char *argv[])
        printf("output data: %d(angle) * %d(range) * %d(scan) * 4 bytes\n",
                YDIM,ZDIM,tdim);
        printf("             3D(9) :ze,rain,wc,qp,vt,vt0,w,fqp,lh\n");
-       printf("             2D(12):sfcz,sfcr,avgsfcr,sfcht,bbht,cldt,cldb,flag(6)\n\n");
+       printf("             2D(12):sfcz,sfcr,sfcht,bbht,cldt,cldb,flag(6)\n\n");
 
         /*--- output GrADS ctl file ---*/
         strcpy(c_memo,"2A25 Latent Heating Profile");
-        strcpy(c_var,"ze,rain,wc,qp,vt,vt0,w,fqp,lh,sfcz,sfcr,avgsfcr,sfcht,bbht,cldt,cldb,f_rain,f_PIA,f_rtype,f_BB,f_bottom,f_wtype");
+        strcpy(c_var,"ze,rain,wc,qp,vt,vt0,w,fqp,lh,sfcz,sfcr,sfcht,bbht,cldt,cldb,f_rain,f_PIA,f_rtype,f_BB,f_bottom,f_wtype");
         nx=1;          ny=YDIM;     nz=ZDIM;    nt=tdim;
         xmin=0.0;      ymin=0.0;    zmin=0.0;   tmin=0.0;
         dx=4.2;        dy=4.2;      dz=ZRESO;   dt=1.0;
